#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>
#include "lcd.h"

#define LCD_WIDTH  84
#define LCD_HEIGHT 48

#define UP    0
#define DOWN  1
#define NONE  5

#define UP_PLAYER1		PD0
#define DOWN_PLAYER1 	PD1
#define UP_PLAYER2   	PD4
#define DOWN_PLAYER2	PD5
#define START			PD6

#define INITIAL_SIZE	6
#define BLOCK_SIZE		2


typedef struct {
    uint8_t x;
    uint8_t y;
} coord;

/* directia de miscare (initial spre dreapta) */
uint8_t bar_direction;
uint8_t bar2_direction;

coord bar[INITIAL_SIZE];
coord bar2[INITIAL_SIZE];

coord puck;
coord puck_dir;

uint32_t bar_size;
uint32_t bar2_size;

uint8_t p1_score = 0;
uint8_t p2_score = 0;

/* Initializare butoane */
void init_buttons(void) {

	DDRD &= ~(1 << UP_PLAYER1);
	PORTD |= (1 << UP_PLAYER1);
	
	DDRD &= ~(1 << DOWN_PLAYER1);
	PORTD |= (1 << DOWN_PLAYER1);
	
	DDRD &= ~(1 << UP_PLAYER2);
	PORTD |= (1 << UP_PLAYER2);
	
	DDRD &= ~(1 << DOWN_PLAYER2);
	PORTD |= (1 << DOWN_PLAYER2);
	
	DDRD &= ~(1 << START);
	PORTD |= (1 << START);

	DDRD |= (1 << PD7);
}

/* Verificare butoane apasate */
void check_direction(void) {

    // UP
    if (!(PIND & (1 << UP_PLAYER1))) {
        bar_direction = UP;
    }
    
    // DOWN
    if (!(PIND & (1 << DOWN_PLAYER1))) {
        bar_direction = DOWN;
    }
    
    // UP S2
    if (!(PIND & (1 << UP_PLAYER2))) {
        bar2_direction = UP;
    }
    
    // DOWN S2
    if (!(PIND & (1 << DOWN_PLAYER2))) {
        bar2_direction = DOWN;
    }
}

/* Generare coordonate pentru puc */
coord generate_coord(void) {
    coord c;
    
    c.x = LCD_WIDTH / 3 + (rand() % LCD_WIDTH / 2);
    c.y = LCD_HEIGHT / 3 + (rand() % LCD_HEIGHT / 2);
    
    return c;
}

/* Imaginea initiala */
void initial_screen(void) {

	lcd_clear();
	lcd_update();
	
	lcd_goto_xy(5, 0);
	lcd_str("Bina Dan");
	lcd_goto_xy(5, 1);
	lcd_str("333CB");
	lcd_goto_xy(5, 3);
	lcd_str("Pong PM 2018");
	lcd_goto_xy(5, 4);
	lcd_str("Press BTN to  start");

}

/* Inceperea jocului prin apasare buton START */
void wait_start(void) {

	initial_screen();

	while (1) {
		if (!(PIND & (1 << START))) {
			break;
		}
		initial_screen();
		lcd_delay();	
	}
	
	lcd_delay();
	lcd_clear();
	lcd_update();
}

/* Setarile initiale */
void init_bar(uint32_t seed) {
    uint8_t i;
    
    bar_size = INITIAL_SIZE;
    
	for (i = 0; i < INITIAL_SIZE; ++i) {
		bar[i].x = 8 ;
      	bar[i].y = LCD_HEIGHT / 2 + i * BLOCK_SIZE;
    }

    for (i = 0; i < INITIAL_SIZE; ++i) {
		bar2[i].x = LCD_WIDTH - 8 ;
        bar2[i].y = LCD_HEIGHT / 2 + i * BLOCK_SIZE;

    }
    srand(seed);
    
    /* generare puc */
    puck = generate_coord();
	puck_dir.x = 1;
	puck_dir.y = 1;

}

/* Update pe pozitii la fiecare pas */
void update_game(void) {
	int i;

	/* Situatiile in care trebuie sa modificam pozitia barii */
    switch(bar_direction) {
        case DOWN:
        	if (bar[INITIAL_SIZE - 1].y + 3 < LCD_HEIGHT) {
            	for (i = 0; i < INITIAL_SIZE; ++i) {
					bar[i].y += 1;
		    	}
            }
            break;
        case UP:
        	if (bar[0].y > 1) {
            	for (i = 0; i < INITIAL_SIZE; ++i) {
					bar[i].y -= 1;
		    	}
        	}
            break;

    }
	
	/* Update pozitie bara 2 */
    switch(bar2_direction) {
        case DOWN:
        	if (bar2[INITIAL_SIZE - 1].y + 3 < LCD_HEIGHT) {
            	for (i = 0; i < INITIAL_SIZE; ++i) {
					bar2[i].y += 1;
		    	}
            }
            break;
        case UP:
        	if (bar2[0].y > 1) {
            	for (i = 0; i < INITIAL_SIZE; ++i) {
					bar2[i].y -= 1;
		    	}
        	}
            break;
    }
    
	/* Coliziunea cu marginea superioara si inferioara a ecranului */
	if (puck.y < 3 || puck.y > LCD_HEIGHT - 4)
		puck_dir.y *= -1;
	
	/* Testam coliziunea pucului cu barile */
	if (puck.x - 2 <= bar[0].x && puck.x + 2 >= bar[0].x && 
		puck.y >= bar[0].y && puck.y <= bar[INITIAL_SIZE - 1].y)
		puck_dir.x *= -1;
	
	if (puck.x - 2 <= bar2[0].x && puck.x + 2 >= bar2[0].x &&
		puck.y >= bar2[0].y && puck.y <= bar2[INITIAL_SIZE - 1].y)
		puck_dir.x *= -1;
	
	/* Situatie cand jucatorul 2 inscrie */
	if (puck.x < 7){
		puck = generate_coord();
		
		p2_score++;
		char s1[10];
		sprintf(s1, "%d - %d", p1_score, p2_score);

		lcd_clear();
		lcd_update();
		
		lcd_goto_xy(6, 1);
   		lcd_str("GOL PLAYER 2");
   		lcd_goto_xy(6, 2);
   		lcd_str(s1);
   		lcd_delay();
        lcd_clear();
        _delay_ms(700);
    }

    /* Situatie cand jucatorul 1 inscrie */
	if (puck.x > LCD_WIDTH - 7) {
		puck = generate_coord();
		p1_score++;
		char s1[10];
        sprintf(s1, "%d - %d", p1_score, p2_score);

		lcd_clear();
		lcd_update();
		
		lcd_goto_xy(6, 1);
   		lcd_str("GOL PLAYER 1");
   		lcd_goto_xy(6, 2);
   		lcd_str(s1);
        lcd_delay();
        lcd_clear();
        _delay_ms(700);
    }

	puck.x += puck_dir.x * 2;
	puck.y += puck_dir.y * 2;

	bar_direction = NONE;
	bar2_direction = NONE;
}

/* Sfarsitul jocului. Se apasa BTN pentru o noua runda */
void end_game(char* str) {

	lcd_clear();
	lcd_update();
	
	lcd_goto_xy(10, 1);
	lcd_str(str);
	
	
	lcd_goto_xy(10, 2);
	lcd_str("Press RESET  to play again");
	
	while(1) {
	}
	
}

/* Deseneaza cele 2 bari si pucul */
void draw_game(void) {
    uint32_t i, j, k;
    coord block;
	coord block2;
    
    for (i = 0; i < bar_size; ++i) {
        for (j = bar[i].x; j < bar[i].x + BLOCK_SIZE; j++) {
        	for (k = bar[i].y; k < bar[i].y + BLOCK_SIZE; k++) {
        		lcd_draw_pixel(j, k);
        	}
        }
	}

	for (i = 0; i < bar_size; ++i) {
        for (j = bar2[i].x; j < bar2[i].x + BLOCK_SIZE; j++) {
        	for (k = bar2[i].y; k < bar2[i].y + BLOCK_SIZE; k++) {
        		lcd_draw_pixel(j, k);
        	}
        }
	}
    for (j = puck.x; j < puck.x + BLOCK_SIZE; j++) {
    	for (k = puck.y; k < puck.y + BLOCK_SIZE; k++) {
    		lcd_draw_pixel(j, k);
    	}
    }	
}

int main(void)
{
	lcd_init();
	lcd_clear();
	lcd_update();
	lcd_contrast(0x46);
	
	init_buttons();
	wait_start();
	bar_direction = NONE;
    bar2_direction = NONE;
    init_bar(32);

	while(1) {
		if (!(PIND & (1 << START))) {
			wait_start();
			continue;
		}
		lcd_clear();
		check_direction();
		update_game();
		draw_game();

		lcd_delay();
		lcd_update();

		if (p1_score > 4) {
			end_game("PLAYER1 WIN");
		} else if (p2_score > 4) {
			end_game("PLAYER2 WIN");
		}
	}
	
	return 0;
}